# Generated by Django 5.0.9 on 2024-10-17 07:44

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("wbcommission", "0005_alter_rebate_audit_log"),
    ]

    operations = [
        migrations.AddField(
            model_name="commissionrule",
            name="consider_zero_percent_for_exclusion",
            field=models.BooleanField(
                default=False,
                help_text="If true, the commission rule with percent 0 (initially consider disabled), will be considered for exclusion rule matching",
                verbose_name="Consider 0 percent for exclusion",
            ),
        ),
    ]
