from wbcore.metadata.configs.titles import TitleViewConfig


class CommissionTitleConfig(TitleViewConfig):
    def get_list_title(self) -> str | None:
        return f"{self.view.commission_type} Commission Rules for account {self.view.account}"

    def get_create_title(self) -> str | None:
        return f"Create {self.view.commission_type} Commission rule for account {self.view.account}"
