# Generated by Django 5.2.9 on 2025-12-09 08:50

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('wbcommission', '0008_alter_commission_options_alter_commission_order'),
    ]

    operations = [
        migrations.RenameIndex(
            model_name='rebate',
            new_name='wbcommissio_commiss_432402_idx',
            old_fields=('commission_type', 'date', 'recipient', 'product', 'account'),
        ),
    ]
