# Generated by Django 5.2.9 on 2025-12-16 15:26

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('directory', '0014_alter_entry_relationship_managers_and_more'),
        ('wbcommission', '0009_rename_rebate_commission_type_date_recipient_product_account_wbcommissio_commiss_432402_idx'),
        ('wbcrm', '0019_alter_activity_companies_alter_activity_participants_and_more'),
        ('wbportfolio', '0095_portfolio_keep_only_essential_positions'),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name='rebate',
            unique_together=set(),
        ),
        migrations.AddConstraint(
            model_name='rebate',
            constraint=models.UniqueConstraint(fields=('date', 'recipient', 'account', 'product', 'commission_type'), name='unique_rebate'),
        ),
    ]
