from wbcore.menus import ItemPermission, MenuItem

COMMISSION_MENUITEM = MenuItem(
    label="Commissions",
    endpoint="wbcommission:commission-list",
    permission=ItemPermission(permissions=["wbcommission.view_commission"]),
)
