# Generated by Django 5.2.9 on 2026-01-14 09:21

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('directory', '0015_alter_emailcontact_address_and_more'),
        ('wbcommission', '0010_alter_rebate_unique_together_rebate_unique_rebate'),
    ]

    operations = [
        migrations.AddConstraint(
            model_name='commissionrole',
            constraint=models.UniqueConstraint(fields=('person', 'commission'), name='unique_commission_role'),
        ),
    ]
